@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (C) 2005-2007 Thomas Perl <thp at perli.net>
+# Copyright (c) 2005-2008 Thomas Perl and the gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -27,6 +27,7 @@
 import gtk
 import pango
 
+import gpodder
 from gpodder import util
 from gpodder.liblogger import log
 
@@ -36,11 +37,18 @@ import time
 import threading
 import ConfigParser
 
+if gpodder.interface == gpodder.MAEMO:
+    default_bittorrent_dir = '/media/mmc2/gpodder/torrents'
+    default_download_dir = '/media/mmc2/gpodder/downloads'
+else:
+    default_bittorrent_dir = os.path.expanduser('~/gpodder-downloads/torrents')
+    default_download_dir = os.path.expanduser('~/gpodder-downloads')
+
 gPodderSettings = {
     # General settings
     'player': ( str, 'xdg-open' ),
     'videoplayer': (str, 'unspecified'),
-    'opml_url': ( str, 'http://gpodder.berlios.de/directory.opml' ),
+    'opml_url': ( str, 'http://www.gpodder.org/directory.opml' ),
     'http_proxy': ( str, '' ),
     'ftp_proxy': ( str, '' ),
     'custom_sync_name': ( str, '{episode.basename}' ),
@@ -49,7 +57,7 @@ gPodderSettings = {
     'max_downloads_enabled': ( bool, False ), 
     'limit_rate': ( bool, False ),
     'limit_rate_value': ( float, 500.0 ),
-    'bittorrent_dir': ( str, os.path.expanduser( '~/gpodder-downloads/torrents') ),
+    'bittorrent_dir': (str, default_bittorrent_dir),
     'episode_old_age': ( int, 7 ),
 
     # Boolean config flags
@@ -89,13 +97,20 @@ gPodderSettings = {
     'ipod_mount': ( str, '/media/ipod' ),
     'mp3_player_folder': ( str, '/media/usbdisk' ),
     'device_type': ( str, 'none' ),
-    'download_dir': ( str, os.path.expanduser( '~/gpodder-downloads') ),
+    'download_dir': (str, default_download_dir),
 
     # Special settings (not in preferences)
     'default_new': ( int, 1 ),
     'use_si_units': ( bool, False ),
     'on_quit_systray': (bool, False),
     'create_m3u_playlists': (bool, False),
+    'max_episodes_per_feed': (int, 200),
+    'mp3_player_use_scrobbler_log': (bool, False),
+    'show_podcast_url_entry': (bool, True),
+    'maemo_allow_custom_player': (bool, False),
+
+    # Hide the cover/pill from the podcast sidebar when it gets too small
+    'podcast_sidebar_save_space': (bool, True),
 
     # Window and paned positions
     'main_window_x': ( int, 100 ),
@@ -107,6 +122,9 @@ gPodderSettings = {
 
 class Config(dict):
     Settings = gPodderSettings
+    
+    # Number of seconds after which settings are auto-saved
+    WRITE_TO_DISK_TIMEOUT = 60
 
     def __init__( self, filename = 'gpodder.conf'):
         dict.__init__( self)
@@ -114,6 +132,7 @@ class Config(dict):
         self.__filename = filename
         self.__section = 'gpodder-conf-1'
         self.__ignore_window_events = False
+        self.__observers = []
         # Name, Type, Value, Type(python type), Editable?, Font weight
         self.__model = gtk.ListStore(str, str, str, object, bool, int)
 
@@ -128,6 +147,22 @@ class Config(dict):
         else:
             raise AttributeError
 
+    def add_observer(self, callback):
+        """
+        Add a callback function as observer. This callback
+        will be called when a setting changes. It should 
+        have this signature:
+
+            observer(name, old_value, new_value)
+
+        The "name" is the setting name, the "old_value" is
+        the value that has been overwritten with "new_value".
+        """
+        if callback not in self.__observers:
+            self.__observers.append(callback)
+        else:
+            log('Observer already added: %s', repr(callback), sender=self)
+
     def connect_gtk_editable( self, name, editable):
         if name in self.Settings:
             editable.delete_text( 0, -1)
@@ -208,18 +243,18 @@ class Config(dict):
             self.__save_thread.start()
 
     def save_thread_proc( self):
-        for i in range( 100):
-            if self.__save_thread != None:
+        for i in range(self.WRITE_TO_DISK_TIMEOUT*10):
+            if self.__save_thread is not None:
                 time.sleep( .1)
-        if self.__save_thread != None:
+        if self.__save_thread is not None:
             self.save()
 
     def __atexit( self):
-        if self.__save_thread != None:
+        if self.__save_thread is not None:
             self.save()
 
     def save( self, filename = None):
-        if filename != None:
+        if filename is not None:
             self.__filename = filename
 
         log( 'Flushing settings to disk', sender = self)
@@ -238,7 +273,7 @@ class Config(dict):
         self.__save_thread = None
 
     def load( self, filename = None):
-        if filename != None:
+        if filename is not None:
             self.__filename = filename
 
         self.__model.clear()
@@ -313,7 +348,14 @@ class Config(dict):
             try:
                 if self[name] != fieldtype(value):
                     log( 'Update: %s = %s', name, value, sender = self)
+                    old_value = self[name]
                     self[name] = fieldtype(value)
+                    for observer in self.__observers:
+                        try:
+                            # Notify observer about config change
+                            observer(name, old_value, self[name])
+                        except:
+                            log('Error while calling observer: %s', repr(observer), sender=self)
                     for row in self.__model:
                         if row[0] == name:
                             row[2] = str(fieldtype(value))
