@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (C) 2005-2007 Thomas Perl <thp at perli.net>
+# Copyright (c) 2005-2008 Thomas Perl and the gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -33,8 +33,8 @@ import urllib
 
 
 def list_channels():
-    for channel in load_channels( load_items = False):
-        msg( 'channel', urllib.unquote( channel.url))
+    for channel in load_channels(load_items=False):
+        msg('podcast', urllib.unquote(channel.url))
 
 
 def add_channel( url):
@@ -43,6 +43,7 @@ def add_channel( url):
     url = util.normalize_feed_url( url)
     try:
         channel = podcastChannel.get_by_url( url, force_update = True)
+        podcastChannel.sync_cache()
     except:
         msg( 'error', _('Could not load feed from URL: %s'), urllib.unquote( url))
         return
@@ -56,7 +57,7 @@ def add_channel( url):
         save_channels( channels)
         msg( 'add', urllib.unquote( url))
     else:
-        msg( 'error', _('Could not add channel.'))
+        msg('error', _('Could not add podcast.'))
 
 
 def del_channel( url):
@@ -73,7 +74,7 @@ def del_channel( url):
     if len(keep_channels) < len(channels):
         save_channels( keep_channels)
     else:
-        msg( 'error', _('Could not remove channel.'))
+        msg('error', _('Could not remove podcast.'))
 
 
 def update():
@@ -111,7 +112,7 @@ def sync_device():
 
     for channel in load_channels():
         if not channel.sync_to_devices:
-            msg('info', _('Skipping channel: %s') % channel.title)
+            msg('info', _('Skipping podcast: %s') % channel.title)
             continue
         
         episodes_to_sync = []
