@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (C) 2005-2007 Thomas Perl <thp at perli.net>
+# Copyright (c) 2005-2008 Thomas Perl and the gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -82,8 +82,8 @@ class DownloadThread(threading.Thread):
         self.filename = self.episode.local_filename()
         self.tempname = os.path.join( os.path.dirname( self.filename), '.tmp-' + os.path.basename( self.filename))
 
-        self.limit_rate = gl.config.limit_rate
-        self.limit_rate_value = gl.config.limit_rate_value
+        # Make an educated guess about the total file size
+        self.total_size = self.episode.length
 
         self.cancelled = False
         self.start_time = 0.0
@@ -92,12 +92,22 @@ class DownloadThread(threading.Thread):
         self.downloader = DownloadURLOpener( self.channel)
         self.last_update = 0.0
 
+        # Keep a copy of these global variables for comparison later        
+        self.limit_rate_value = gl.config.limit_rate_value
+        self.limit_rate = gl.config.limit_rate
+        self.start_blocks = 0
+
     def cancel( self):
         self.cancelled = True
 
     def status_updated( self, count, blockSize, totalSize):
         if totalSize:
             self.progress = 100.0*float(count*blockSize)/float(totalSize)
+            # We see a different "total size" while downloading,
+            # so correct the total size variable in the thread
+            if totalSize != self.total_size:
+                log('Correcting file size for %s from %d to %d while downloading.', self.url, self.total_size, totalSize, sender=self)
+                self.total_size = totalSize
         else:
             self.progress = 100.0
 
@@ -114,21 +124,38 @@ class DownloadThread(threading.Thread):
         if count % 5 == 0:
             now = time.time()
             if self.start_time > 0:
+                
+                # Has rate limiting been enabled or disabled?                
+                if self.limit_rate != gl.config.limit_rate: 
+                    # If it has been enabled then reset base time and block count                    
+                    if gl.config.limit_rate:
+                        self.start_time = now
+                        self.start_blocks = count
+                    self.limit_rate = gl.config.limit_rate
+                    
+                # Has the rate been changed and are we currently limiting?            
+                if self.limit_rate_value != gl.config.limit_rate_value and self.limit_rate: 
+                    self.start_time = now
+                    self.start_blocks = count
+                    self.limit_rate_value = gl.config.limit_rate_value
+
                 passed = now - self.start_time
                 if passed > 0:
-                    speed = (count*blockSize)/passed
+                    speed = ((count-self.start_blocks)*blockSize)/passed
                 else:
                     speed = 0
             else:
                 self.start_time = now
+                self.start_blocks = count
                 passed = now - self.start_time
                 speed = count*blockSize
+
             self.speed = '%s/s' % gl.format_filesize(speed)
 
-            if self.limit_rate and speed > self.limit_rate_value:
+            if gl.config.limit_rate and speed > gl.config.limit_rate_value:
                 # calculate the time that should have passed to reach
                 # the desired download rate and wait if necessary
-                should_have_passed = float(count*blockSize)/(self.limit_rate_value*1024.0)
+                should_have_passed = float((count-self.start_blocks)*blockSize)/(gl.config.limit_rate_value*1024.0)
                 if should_have_passed > passed:
                     # sleep a maximum of 10 seconds to not cause time-outs
                     delay = min( 10.0, float(should_have_passed-passed))
@@ -156,9 +183,9 @@ class DownloadThread(threading.Thread):
                 services.download_status_manager.remove_download_id( self.download_id)
                 services.download_status_manager.s_release( acquired)
         except DownloadCancelledException:
-            log( 'Download has been cancelled: %s', self.episode.title, sender = self)
+            log('Download has been cancelled: %s', self.episode.title, traceback=None, sender=self)
         except IOError, ioe:
-            if self.notification != None:
+            if self.notification is not None:
                 title = ioe.strerror
                 message = _('An error happened while trying to download <b>%s</b>.') % ( saxutils.escape( self.episode.title), )
                 self.notification( message, title)
