@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (C) 2005-2007 Thomas Perl <thp at perli.net>
+# Copyright (c) 2005-2008 Thomas Perl and the gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -31,6 +31,7 @@ import urllib
 import shutil
 import xml.dom.minidom
 
+import gpodder
 from gpodder import util
 from gpodder import opml
 from gpodder import config
@@ -45,10 +46,17 @@ from liblogger import log
 
 import shlex
 
+if gpodder.interface == gpodder.MAEMO:
+    import osso
+
 class gPodderLib(object):
     def __init__( self):
         log('Creating gPodderLib()', sender=self)
-        gpodder_dir = os.path.expanduser( '~/.config/gpodder/')
+        if gpodder.interface == gpodder.MAEMO:
+            gpodder_dir = '/media/mmc2/gpodder/'
+            self.osso_c = osso.Context('gpodder_osso_sender', '1.0', False)
+        else:
+            gpodder_dir = os.path.expanduser('~/.config/gpodder/')
         util.make_directory( gpodder_dir)
 
         self.tempdir = gpodder_dir
@@ -191,11 +199,29 @@ class gPodderLib(object):
 
     def history_is_locked( self, url):
         return (url in self.__locked_history)
+    
+    def send_subscriptions(self):
+        try:
+            subprocess.Popen(['xdg-email', '--subject', _('My podcast subscriptions'),
+                                           '--attach', self.channel_opml_file])
+        except:
+            return False
+        
+        return True
 
     def playback_episode( self, channel, episode):
         self.history_mark_played( episode.url)
         filename = episode.local_filename()
 
+        if gpodder.interface == gpodder.MAEMO and not self.config.maemo_allow_custom_player:
+            # Use the built-in Nokia Mediaplayer here
+            filename = filename.encode('utf-8')
+            osso_rpc = osso.Rpc(self.osso_c)
+            service = 'com.nokia.mediaplayer'
+            path = '/com/nokia/mediaplayer'
+            osso_rpc.rpc_run(service, path, service, 'mime_open', ('file://'+filename,))
+            return (True, service)
+
         # Determine the file type and set the player accordingly.  
         file_type = util.file_type_by_extension(util.file_extension_from_url(episode.url))
 
@@ -216,21 +242,21 @@ class gPodderLib(object):
         return ( True, command_line[0] )
 
     def image_download_thread( self, url, callback_pixbuf = None, callback_status = None, callback_finished = None, cover_file = None):
-        if callback_status != None:
-            util.idle_add(callback_status, _('Downloading channel cover...'))
+        if callback_status is not None:
+            util.idle_add(callback_status, _('Downloading podcast cover...'))
         pixbuf = gtk.gdk.PixbufLoader()
         
         if cover_file is None:
             log( 'Downloading %s', url)
             pixbuf.write( urllib.urlopen(url).read())
         
-        if cover_file != None and not os.path.exists( cover_file):
+        if cover_file is not None and not os.path.exists(cover_file):
             log( 'Downloading cover to %s', cover_file)
             cachefile = open( cover_file, "w")
             cachefile.write( urllib.urlopen(url).read())
             cachefile.close()
         
-        if cover_file != None:
+        if cover_file is not None:
             log( 'Reading cover from %s', cover_file)
             try:
                 pixbuf.write( open( cover_file, "r").read())
@@ -246,7 +272,7 @@ class gPodderLib(object):
             util.delete_file( cover_file)
         
         MAX_SIZE = 400
-        if callback_pixbuf != None:
+        if callback_pixbuf is not None:
             pb = pixbuf.get_pixbuf()
             if pb:
                 if pb.get_width() > MAX_SIZE:
@@ -256,9 +282,9 @@ class gPodderLib(object):
                     factor = MAX_SIZE*1.0/pb.get_height()
                     pb = pb.scale_simple( int(pb.get_width()*factor), int(pb.get_height()*factor), gtk.gdk.INTERP_BILINEAR)
                 util.idle_add(callback_pixbuf, pb)
-        if callback_status != None:
+        if callback_status is not None:
             util.idle_add(callback_status, '')
-        if callback_finished != None:
+        if callback_finished is not None:
             util.idle_add(callback_finished)
 
     def get_image_from_url( self, url, callback_pixbuf = None, callback_status = None, callback_finished = None, cover_file = None):
