@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (C) 2005-2007 Thomas Perl <thp at perli.net>
+# Copyright (c) 2005-2008 Thomas Perl and the gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -27,22 +27,22 @@
 #            liblocdbreader.py (2006-01-10)
 #
 
-
 import gtk
 import gobject
 import pango
 
+import gpodder
 from gpodder import util
 from gpodder import opml
 from gpodder import cache
 from gpodder import services
 from gpodder import draw
+from gpodder import libtagupdate
+from gpodder import dumbshelve
 
 from gpodder.liblogger import log
 from gpodder.libgpodder import gl
 
-from os.path import exists
-from os.path import basename
 import os.path
 import os
 import glob
@@ -52,31 +52,34 @@ import urllib
 import urlparse
 import time
 import threading
-
-from datetime import datetime
-
-from libtagupdate import update_metadata_on_file
-from libtagupdate import tagging_supported
-
-from threading import Event
-import re
-
-from types import ListType
-from email.Utils import mktime_tz
-from email.Utils import parsedate_tz
-
-from xml.sax import saxutils
-
+import datetime
+import md5
 import xml.dom.minidom
 
-import md5
+from email.Utils import mktime_tz
+from email.Utils import parsedate_tz
+from xml.sax import saxutils
 
-import string
-
-from gpodder import dumbshelve
 
 global_lock = threading.RLock()
 
+
+if gpodder.interface == gpodder.MAEMO:
+    ICON_AUDIO_FILE = 'gnome-mime-audio-mp3'
+    ICON_VIDEO_FILE = 'gnome-mime-video-mp4'
+    ICON_BITTORRENT = 'qgn_toolb_browser_web'
+    ICON_DOWNLOADING = 'qgn_toolb_messagin_moveto'
+    ICON_DELETED = 'qgn_toolb_gene_deletebutton'
+    ICON_NEW = 'qgn_list_gene_favor'
+else:
+    ICON_AUDIO_FILE = 'audio-x-generic'
+    ICON_VIDEO_FILE = 'video-x-generic'
+    ICON_BITTORRENT = 'applications-internet'
+    ICON_DOWNLOADING = gtk.STOCK_GO_DOWN
+    ICON_DELETED = gtk.STOCK_DELETE
+    ICON_NEW = gtk.STOCK_NEW
+
+
 class ChannelSettings(object):
     storage = dumbshelve.open_shelve(gl.channel_settings_file)
 
@@ -115,9 +118,9 @@ class EpisodeURLMetainfo(object):
             return result
 
 
-class podcastChannel(ListType):
+class podcastChannel(list):
     """holds data for a complete channel"""
-    SETTINGS = ('sync_to_devices', 'is_music_channel', 'device_playlist_name','override_title','username','password')
+    SETTINGS = ('sync_to_devices', 'device_playlist_name','override_title','username','password')
     icon_cache = {}
 
     storage = dumbshelve.open_shelve(gl.feed_cache_file)
@@ -131,53 +134,71 @@ class podcastChannel(ListType):
                 del cls.storage[url]
 
     @classmethod
-    def get_by_url( cls, url, force_update = False, offline = False):
+    def sync_cache(cls):
+        cls.storage.sync()
+
+    @classmethod
+    def get_by_url(cls, url, force_update=False, offline=False, default_title=None, old_channel=None):
         if isinstance( url, unicode):
             url = url.encode('utf-8')
 
-        c = cls.fc.fetch( url, force_update, offline)
+        (updated, c) = cls.fc.fetch( url, force_update, offline)
+        # If we have an old instance of this channel, and
+        # feedcache says the feed hasn't changed, return old
+        if not updated and old_channel:
+            log('using old channel for %s', url)
+            return old_channel
+
         channel = podcastChannel( url)
+        channel.parse_error = c.get('bozo_exception', None)
         channel.load_settings()
-        channel.title = c.feed.title
+        if hasattr(c.feed, 'title'):
+            channel.title = c.feed.title
+        elif default_title is not None:
+            channel.title = default_title
+        else:
+            channel.title = url
         if hasattr( c.feed, 'link'):
             channel.link = c.feed.link
         if hasattr( c.feed, 'subtitle'):
-            channel.description = util.remove_html_tags( c.feed.subtitle)
+            channel.description = util.remove_html_tags(c.feed.subtitle)
 
-        if hasattr( c.feed, 'updated_parsed'):
-            channel.pubDate = util.updated_parsed_to_rfc2822( c.feed.updated_parsed)
+        if hasattr(c.feed, 'updated_parsed') and c.feed.updated_parsed is not None:
+            channel.pubDate = util.updated_parsed_to_rfc2822(c.feed.updated_parsed)
         if hasattr( c.feed, 'image'):
             if c.feed.image.href:
                 channel.image = c.feed.image.href
 
-        for entry in c.entries:
-            if not hasattr( entry, 'enclosures'):
-                log('Skipping entry: %s', entry.get( 'id', '(no id available)'), sender = channel)
-                continue
-
+        # We can limit the maximum number of entries that gPodder will parse
+        # via the "max_episodes_per_feed" configuration option.
+        if len(c.entries) > gl.config.max_episodes_per_feed:
+            log('Limiting number of episodes for %s to %d', channel.title, gl.config.max_episodes_per_feed)
+        for entry in c.entries[:min(gl.config.max_episodes_per_feed, len(c.entries))]:
             episode = None
 
             try:
                 episode = podcastItem.from_feedparser_entry( entry, channel)
             except:
-                log( 'Cannot instantiate episode: %s. Skipping.', entry.get( 'id', '(no id available)'), sender = channel)
+                log( 'Cannot instantiate episode: %s. Skipping.', entry.get( 'id', '(no id available)'), sender = channel, traceback=True)
 
             if episode:
                 channel.append( episode)
 
         channel.sort( reverse = True)
         
-        cls.storage.sync()
         return channel
 
     @staticmethod
-    def create_from_dict( d, load_items = True, force_update = False, callback_error = None, offline = False):
+    def create_from_dict(d, load_items=True, force_update=False, callback_error=None, offline=False, old_channel=None):
         if load_items:
             try:
-                return podcastChannel.get_by_url( d['url'], force_update = force_update, offline= offline)
+                default_title = None
+                if 'title' in d:
+                    default_title = d['title']
+                return podcastChannel.get_by_url(d['url'], force_update, offline, default_title, old_channel)
             except:
                 callback_error and callback_error( _('Could not load channel feed from URL: %s') % d['url'])
-                log( 'Cannot load podcastChannel from URL: %s', d['url'])
+                log( 'Cannot load podcastChannel from URL: %s', d['url'], traceback=True)
 
         c = podcastChannel()
         for key in ( 'url', 'title', 'description' ):
@@ -194,12 +215,11 @@ class podcastChannel(ListType):
         self.description = util.remove_html_tags( description)
         self.image = None
         self.pubDate = ''
+        self.parse_error = None
 
         # should this channel be synced to devices? (ex: iPod)
         self.sync_to_devices = True
-        # if this is set to true, device syncing (ex: iPod) should treat this as music, not as podcast)
-        self.is_music_channel = False
-        # to which playlist should be synced when "is_music_channel" is true?
+        # to which playlist should be synced
         self.device_playlist_name = 'gPodder'
         # if set, this overrides the channel-provided title
         self.override_title = ''
@@ -353,10 +373,10 @@ class podcastChannel(ListType):
             self.save_downloaded_episodes( downloaded_episodes)
 
             # Update metadata on file (if possible and wanted)
-            if gl.config.update_tags and tagging_supported():
+            if gl.config.update_tags and libtagupdate.tagging_supported():
                 filename = item.local_filename()
                 try:
-                    update_metadata_on_file( filename, title = item.title, artist = self.title)
+                    libtagupdate.update_metadata_on_file(filename, title=item.title, artist=self.title)
                 except:
                     log('Error while calling update_metadata_on_file() :(')
 
@@ -422,7 +442,7 @@ class podcastChannel(ListType):
         self.update_save_dir_size()
 
         iter = self.tree_model.get_iter_first()
-        while iter != None:
+        while iter is not None:
             self.iter_set_downloading_columns( self.tree_model, iter, new_episodes)
             iter = self.tree_model.iter_next( iter)
 
@@ -435,6 +455,9 @@ class podcastChannel(ListType):
         return self.__tree_model
 
     def iter_set_downloading_columns( self, model, iter, new_episodes = []):
+        global ICON_AUDIO_FILE, ICON_VIDEO_FILE, ICON_BITTORRENT
+        global ICON_DOWNLOADING, ICON_DELETED, ICON_NEW
+        
         url = model.get_value( iter, 0)
         local_filename = model.get_value( iter, 8)
         played = not gl.history_is_played(url)
@@ -448,20 +471,20 @@ class podcastChannel(ListType):
         if os.path.exists( local_filename):
             file_type = util.file_type_by_extension( util.file_extension_from_url(url))
             if file_type == 'audio':
-                status_icon = util.get_tree_icon('audio-x-generic', played, locked, self.icon_cache, icon_size)
+                status_icon = util.get_tree_icon(ICON_AUDIO_FILE, played, locked, self.icon_cache, icon_size)
             elif file_type == 'video':
-                status_icon = util.get_tree_icon('video-x-generic', played, locked, self.icon_cache, icon_size)
+                status_icon = util.get_tree_icon(ICON_VIDEO_FILE, played, locked, self.icon_cache, icon_size)
             elif file_type == 'torrent':
-                status_icon = util.get_tree_icon('applications-internet', played, locked, self.icon_cache, icon_size)
+                status_icon = util.get_tree_icon(ICON_BITTORRENT, played, locked, self.icon_cache, icon_size)
             else:
                 status_icon = util.get_tree_icon('unknown', played, locked, self.icon_cache, icon_size)
             
         elif services.download_status_manager.is_download_in_progress(url):
-            status_icon = util.get_tree_icon(gtk.STOCK_GO_DOWN, icon_cache=self.icon_cache, icon_size=icon_size)
+            status_icon = util.get_tree_icon(ICON_DOWNLOADING, icon_cache=self.icon_cache, icon_size=icon_size)
         elif gl.history_is_downloaded(url):
-            status_icon = util.get_tree_icon(gtk.STOCK_DELETE, icon_cache=self.icon_cache, icon_size=icon_size)
+            status_icon = util.get_tree_icon(ICON_DELETED, icon_cache=self.icon_cache, icon_size=icon_size)
         elif url in [e.url for e in new_episodes]:
-            status_icon = util.get_tree_icon(gtk.STOCK_NEW, icon_cache=self.icon_cache, icon_size=icon_size)
+            status_icon = util.get_tree_icon(ICON_NEW, icon_cache=self.icon_cache, icon_size=icon_size)
         else:
             status_icon = None
 
@@ -559,16 +582,24 @@ class podcastItem(object):
         if episode.title == '':
             log( 'Warning: Episode has no title, adding anyways.. (Feed Is Buggy!)', sender = episode)
 
-        enclosure = entry.enclosures[0]
-        if len(entry.enclosures) > 1:
-            for e in entry.enclosures:
-                if hasattr( e, 'href') and hasattr( e, 'length') and hasattr( e, 'type') and (e.type.startswith('audio/') or e.type.startswith('video/')):
-                    if util.normalize_feed_url( e.href) != None:
-                        log( 'Selected enclosure: %s', e.href, sender = episode)
-                        enclosure = e
-                        break
+        enclosure = None
+        if hasattr(entry, 'enclosures') and len(entry.enclosures) > 0:
+            enclosure = entry.enclosures[0]
+            if len(entry.enclosures) > 1:
+                for e in entry.enclosures:
+                    if hasattr( e, 'href') and hasattr( e, 'length') and hasattr( e, 'type') and (e.type.startswith('audio/') or e.type.startswith('video/')):
+                        if util.normalize_feed_url(e.href) is not None:
+                            log( 'Selected enclosure: %s', e.href, sender = episode)
+                            enclosure = e
+                            break
+            episode.url = util.normalize_feed_url( enclosure.get( 'href', ''))
+        elif hasattr(entry, 'link'):
+            extension = util.file_extension_from_url(entry.link)
+            file_type = util.file_type_by_extension(extension)
+            if file_type is not None:
+                log('Adding episode with link to file type "%s".', file_type, sender=episode)
+                episode.url = entry.link
 
-        episode.url = util.normalize_feed_url( enclosure.get( 'href', ''))
         if not episode.url:
             raise ValueError( 'Episode has an invalid URL')
 
@@ -666,7 +697,7 @@ class podcastItem(object):
     @property
     def published( self):
         try:
-            return datetime.fromtimestamp( mktime_tz( parsedate_tz( self.pubDate))).strftime('%Y%m%d')
+            return datetime.datetime.fromtimestamp( mktime_tz( parsedate_tz( self.pubDate))).strftime('%Y%m%d')
         except:
             log( 'Cannot format pubDate for "%s".', self.title, sender = self)
             return '00000000'
@@ -682,7 +713,7 @@ class podcastItem(object):
         except:
             # by default, do as if this is not the same
             # this is here so that comparisons with None 
-            # can be allowed (item != None -> True)
+            # can be allowed (item is not None -> True)
             return -1
         
         return timestamp_self - timestamp_other
@@ -707,21 +738,16 @@ class podcastItem(object):
             return pubdate
 
     def cute_pubdate( self):
-        seconds_in_a_day = 86400
         try:
-            timestamp = int(mktime_tz( parsedate_tz( self.pubDate)))
+            timestamp = int(mktime_tz(parsedate_tz(self.pubDate)))
         except:
-            return _("(unknown)")
-        diff = int((time.time()+1)/seconds_in_a_day) - int(timestamp/seconds_in_a_day)
+            timestamp = None
         
-        if diff == 0:
-           return _("Today")
-        if diff == 1:
-           return _("Yesterday")
-        if diff < 7:
-            return str(datetime.fromtimestamp( timestamp).strftime( "%A"))
-        
-        return str(datetime.fromtimestamp( timestamp).strftime( "%x"))
+        result = util.format_date(timestamp)
+        if result is None:
+            return '(%s)' % _('unknown')
+        else:
+            return result
     
     pubdate_prop = property(fget=cute_pubdate)
 
@@ -758,7 +784,7 @@ class podcastItem(object):
 
 
 def channels_to_model(channels):
-    new_model = gtk.ListStore(str, str, str, gtk.gdk.Pixbuf, int, gtk.gdk.Pixbuf)
+    new_model = gtk.ListStore(str, str, str, gtk.gdk.Pixbuf, int, gtk.gdk.Pixbuf, str)
     
     for channel in channels:
         (count_available, count_downloaded, count_new, count_unplayed) = channel.get_episode_stats()
@@ -769,7 +795,14 @@ def channels_to_model(channels):
 
         title_markup = saxutils.escape(channel.title)
         description_markup = saxutils.escape(util.get_first_line(channel.description))
-        new_model.set(new_iter, 2, '%s\n<small>%s</small>' % (title_markup, description_markup))
+        description = '%s\n<small>%s</small>' % (title_markup, description_markup)
+        if channel.parse_error is not None:
+            description = '<span foreground="#ff0000">%s</span>' % description
+            new_model.set(new_iter, 6, channel.parse_error)
+        else:
+            new_model.set(new_iter, 6, '')
+        
+        new_model.set(new_iter, 2, description)
 
         if count_unplayed > 0 or count_downloaded > 0:
             new_model.set(new_iter, 3, draw.draw_pill_pixbuf(str(count_unplayed), str(count_downloaded)))
@@ -805,20 +838,37 @@ def channels_to_model(channels):
 
 
 
-def load_channels( load_items = True, force_update = False, callback_proc = None, callback_url = None, callback_error = None, offline = False):
+def load_channels(load_items=True, force_update=False, callback_proc=None, callback_url=None, callback_error=None, offline=False, is_cancelled_cb=None, old_channels=None):
     importer = opml.Importer(gl.channel_opml_file)
     result = []
+    if old_channels is None:
+        old_channels = {}
+    else:
+        # Convert list of channels to a dict with URLs as keys
+        old_channels = dict(map(lambda c: (c.url, c), old_channels))
 
     urls_to_keep = []
     count = 0
     for item in importer.items:
+        if is_cancelled_cb is not None:
+            cancelled = is_cancelled_cb()
+            if cancelled:
+                # We don't force updates for all upcoming episodes
+                force_update = False
+                offline = True
         callback_proc and callback_proc( count, len( importer.items))
         callback_url and callback_url( item['url'])
         urls_to_keep.append(item['url'])
-        result.append( podcastChannel.create_from_dict( item, load_items = load_items, force_update = force_update, callback_error = callback_error, offline = offline))
+        if item['url'] not in old_channels:
+            old_channel = None
+        else:
+            old_channel = old_channels[item['url']]
+        channel = podcastChannel.create_from_dict(item, load_items, force_update, callback_error, offline, old_channel)
+        result.append(channel)
         count += 1
 
     podcastChannel.clear_cache(urls_to_keep)
+    podcastChannel.sync_cache()
     result.sort(key=lambda x:x.title.lower())
     return result
 
