@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (C) 2005-2007 Thomas Perl <thp at perli.net>
+# Copyright (c) 2005-2008 Thomas Perl and the gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -78,17 +78,50 @@ class DownloadStatusManager(ObservableService):
         self.last_progress_status  = ( 0, 0 )
         
         # use to correctly calculate percentage done
-        self.downloads_done_count = 0
+        self.downloads_done_bytes = 0
         
         self.max_downloads = gl.config.max_downloads
         self.semaphore = threading.Semaphore( self.max_downloads)
 
         self.tree_model = gtk.ListStore( *self.COLUMN_TYPES)
         self.tree_model_lock = threading.Lock()
+
+        # batch add in progress?
+        self.batch_mode_enabled = False
+        # we set this flag if we would notify inside batch mode
+        self.batch_mode_notify_flag = False
+
+        # Used to notify all threads that they should
+        # re-check if they can acquire the lock
+        self.notification_event = threading.Event()
+        self.notification_event_waiters = 0
         
         signal_names = ['list-changed', 'progress-changed', 'progress-detail', 'download-complete']
         ObservableService.__init__(self, signal_names)
 
+    def start_batch_mode(self):
+        """
+        This is called when we are going to add multiple
+        episodes to our download list, and do not want to
+        notify the GUI for every single episode.
+
+        After all episodes have been added, you MUST call
+        the end_batch_mode() method to trigger a notification.
+        """
+        self.batch_mode_enabled = True
+
+    def end_batch_mode(self):
+        """
+        This is called after multiple episodes have been
+        added when start_batch_mode() has been called before.
+
+        This sends out a notification that the list has changed.
+        """
+        self.batch_mode_enabled = False
+        if self.batch_mode_notify_flag:
+            self.notify('list-changed')
+        self.batch_mode_notify_flag = False
+
     def notify_progress( self):
         now = ( self.count(), self.average_progress() )
         
@@ -99,18 +132,41 @@ class DownloadStatusManager(ObservableService):
     def s_acquire( self):
         if not gl.config.max_downloads_enabled:
             return False
-
-        # Release queue slots if user has enabled more slots
-        while self.max_downloads < gl.config.max_downloads:
-            self.semaphore.release()
-            self.max_downloads += 1
-
+        
         # Acquire queue slots if user has decreased the slots
         while self.max_downloads > gl.config.max_downloads:
             self.semaphore.acquire()
             self.max_downloads -= 1
 
-        return self.semaphore.acquire()
+        # Make sure we update the maximum number of downloads
+        self.update_max_downloads()
+
+        while self.semaphore.acquire(False) == False:
+            self.notification_event_waiters += 1
+            self.notification_event.wait(2.)
+            self.notification_event_waiters -= 1
+
+            # If we are the last thread that woke up from
+            # the notification_event, clear the flag here
+            if self.notification_event_waiters == 0:
+                self.notification_event.clear()
+                
+            # If the user has change the config option since the
+            # last time we checked, return false and start download
+            if not gl.config.max_downloads_enabled:
+                return False
+
+        # If we land here, we've acquired exactly the one we need
+        return True
+    
+    def update_max_downloads(self):
+        # Release queue slots if user has enabled more slots
+        while self.max_downloads < gl.config.max_downloads:
+            self.semaphore.release()
+            self.max_downloads += 1
+
+        # Notify all threads that the limit might have been changed
+        self.notification_event.set()
 
     def s_release( self, acquired = True):
         if acquired:
@@ -128,14 +184,17 @@ class DownloadStatusManager(ObservableService):
     def register_download_id( self, id, thread):
         self.tree_model_lock.acquire()
         self.status_list[id] = { 'iter': self.tree_model.append(), 'thread': thread, 'progress': 0.0, 'speed': _('Queued'), }
-        self.notify( 'list-changed')
+        if self.batch_mode_enabled:
+            self.batch_mode_notify_flag = True
+        else:
+            self.notify('list-changed')
         self.tree_model_lock.release()
 
     def remove_download_id( self, id):
         if not id in self.status_list:
             return
         iter = self.status_list[id]['iter']
-	if iter != None:
+	if iter is not None:
             self.tree_model_lock.acquire()
             util.idle_add(self.remove_iter, iter)
             self.tree_model_lock.release()
@@ -144,8 +203,11 @@ class DownloadStatusManager(ObservableService):
             del self.status_list[id]
             if not self.has_items():
                 # Reset the counter now
-                self.downloads_done_count = 0
-        self.notify( 'list-changed')
+                self.downloads_done_bytes = 0
+        if self.batch_mode_enabled:
+            self.batch_mode_notify_flag = True
+        else:
+            self.notify('list-changed')
         self.notify_progress()
 
     def count( self):
@@ -158,8 +220,11 @@ class DownloadStatusManager(ObservableService):
         if not len(self.status_list):
             return 0
 
-        done = self.downloads_done_count*100 + sum([status['progress'] for status in self.status_list.values()])
-        return done / (len(self.status_list) + self.downloads_done_count)
+        done = sum(status['progress']/100. * status['thread'].total_size for status in self.status_list.values())
+        total = sum(status['thread'].total_size for status in self.status_list.values())
+        if total + self.downloads_done_bytes == 0:
+            return 0
+        return float(done + self.downloads_done_bytes) / float(total + self.downloads_done_bytes) * 100
 
     def update_status( self, id, **kwargs):
         if not id in self.status_list:
@@ -182,7 +247,7 @@ class DownloadStatusManager(ObservableService):
     def download_completed(self, id):
         if id in self.status_list:
             self.notify('download-complete', self.status_list[id]['episode'])
-            self.downloads_done_count += 1
+            self.downloads_done_bytes += self.status_list[id]['thread'].total_size
 
     def request_progress_detail( self, url):
         for status in self.status_list.values():
@@ -190,12 +255,18 @@ class DownloadStatusManager(ObservableService):
                 self.notify( 'progress-detail', url, status['progress'], status['speed'])
 
     def is_download_in_progress( self, url):
-        for element in self.status_list:
-	    thread = self.status_list[element]['thread']
-	    if thread != None and thread.url == url:
-	        return True
-	
-	return False
+        for element in self.status_list.keys():
+            # We need this, because status_list is modified from other threads
+            if element in self.status_list:
+                try:
+                    thread = self.status_list[element]['thread']
+                except:
+                    thread = None
+
+                if thread is not None and thread.url == url:
+                    return True
+        
+        return False
 
     def cancel_all( self):
         for element in self.status_list:
@@ -203,12 +274,12 @@ class DownloadStatusManager(ObservableService):
 	    self.status_list[element]['thread'].cancel()
         # clear the tree model after cancelling
         util.idle_add(self.tree_model.clear)
-        self.downloads_done_count = 0
+        self.downloads_done_bytes = 0
 
     def cancel_by_url( self, url):
         for element in self.status_list:
 	    thread = self.status_list[element]['thread']
-	    if thread != None and thread.url == url:
+	    if thread is not None and thread.url == url:
                 self.remove_download_id( element)
 		return True
         
