@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (C) 2005-2007 Thomas Perl <thp at perli.net>
+# Copyright (c) 2005-2008 Thomas Perl and the gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -93,11 +93,35 @@ def get_track_length(filename):
 
 
 class SyncTrack(object):
+    """
+    This represents a track that is on a device. You need
+    to specify at least the following keyword arguments, 
+    because these will be used to display the track in the
+    GUI. All other keyword arguments are optional and can
+    be used to reference internal objects, etc... See the
+    iPod synchronization code for examples.
+
+    Keyword arguments needed:
+        playcount (How often has the track been played?)
+        podcast (Which podcast is this track from? Or: Folder name)
+        released (The release date of the episode)
+
+    If any of these fields is unknown, it should not be
+    passed to the function (the values will default to None
+    for all required fields).
+    """
     def __init__(self, title, length, modified, **kwargs):
         self.title = title
         self.length = length
         self.filesize = util.format_filesize(length, gl.config.use_si_units)
         self.modified = modified
+
+        # Set some (possible) keyword arguments to default values
+        self.playcount = None
+        self.podcast = None
+        self.released = None
+
+        # Convert keyword arguments to object attributes
         self.__dict__.update(kwargs)
 
 
@@ -230,8 +254,10 @@ class iPodDevice(Device):
 
             age_in_days = util.file_age_in_days(filename)
             modified = util.file_age_to_string(age_in_days)
+            released = gpod.itdb_time_mac_to_host(track.time_released)
+            released = util.format_date(released)
 
-            t = SyncTrack(track.title, length, modified, libgpodtrack=track)
+            t = SyncTrack(track.title, length, modified, libgpodtrack=track, playcount=track.playcount, released=released, podcast=track.artist)
             tracks.append(t)
         return tracks
 
@@ -390,6 +416,15 @@ class iPodDevice(Device):
 
 
 class MP3PlayerDevice(Device):
+    # if different players use other filenames besides
+    # .scrobbler.log, add them to this list
+    scrobbler_log_filenames = ['.scrobbler.log']
+
+    # This is the maximum length of a file name that is
+    # created on the MP3 player, because FAT32 has a
+    # 255-character limit for the whole path
+    MAX_FILENAME_LENGTH = 100
+
     def __init__(self):
         Device.__init__(self)
 
@@ -408,21 +443,35 @@ class MP3PlayerDevice(Device):
 
         self.destination = gl.config.mp3_player_folder
         self.buffer_size = 1024*1024 # 1 MiB
+        self.scrobbler_log = []
 
     def open(self):
         self.notify('status', _('Opening MP3 player'))
         if util.directory_is_writable(self.destination):
             self.notify('status', _('MP3 player opened'))
+            if gl.config.mp3_player_use_scrobbler_log:
+                mp3_player_mount_point = util.find_mount_point(self.destination)
+                # If a moint point cannot be found look inside self.destination for scrobbler_log_filenames
+                # this prevents us from os.walk()'ing the entire / filesystem
+                if mp3_player_mount_point == '/':
+                    mp3_player_mount_point = self.destination
+                log_location = self.find_scrobbler_log(mp3_player_mount_point)
+                if log_location is not None and self.load_audioscrobbler_log(log_location):
+                    log('Using Audioscrobbler log data to mark tracks as played', sender=self)
             return True
         else:
             return False
-    
+
     def add_track(self, episode):
         self.notify('status', _('Adding %s') % episode.title)
 
         if gl.config.fssync_channel_subfolders:
             # Add channel title as subfolder
             folder = episode.channel.title
+            # Don't allow extremely long folder names (filesystem limitations)
+            if len(folder) > self.MAX_FILENAME_LENGTH:
+                log('Limiting folder "%s" to %d characters.', folder, self.MAX_FILENAME_LENGTH, sender=self)
+                folder = folder[:self.MAX_FILENAME_LENGTH]
             folder = re.sub('[/|?*<>:+\[\]\"\\\]', '_', folder.encode(self.enc, 'ignore'))
             folder = os.path.join(self.destination, folder)
         else:
@@ -432,10 +481,10 @@ class MP3PlayerDevice(Device):
 
         filename_base = episode.sync_filename()
 
-        # don't allow extremely long file names: cut the
-        # filename at 50 chars (to make FAT-based drives happy)
-        if len(filename_base) > 50:
-            filename_base = filename_base[:50]
+        # don't allow extremely long file names (filesystem limitations)
+        if len(filename_base) > self.MAX_FILENAME_LENGTH:
+            log('Limiting filename "%s" to %d characters.', filename_base, self.MAX_FILENAME_LENGTH, sender=self)
+            filename_base = filename_base[:self.MAX_FILENAME_LENGTH]
 
         to_file = filename_base + os.path.splitext(from_file)[1].lower()
 
@@ -459,6 +508,11 @@ class MP3PlayerDevice(Device):
                 log('Cannot create folder on MP3 player: %s', folder, sender=self)
                 return False
 
+        if (gl.config.mp3_player_use_scrobbler_log and not episode.is_played()
+                and [episode.channel.title, episode.title] in self.scrobbler_log):
+            log('Marking "%s" from "%s" as played', episode.title, episode.channel.title, sender=self)
+            gl.history_mark_played(episode.url)
+
         if not os.path.exists(to_file):
             log('Copying %s => %s', os.path.basename(from_file), to_file.decode(self.enc), sender=self)
             return self.copy_file_progress(from_file, to_file)
@@ -525,8 +579,12 @@ class MP3PlayerDevice(Device):
          
             age_in_days = util.file_age_in_days(filename)
             modified = util.file_age_to_string(age_in_days)
+            if gl.config.fssync_channel_subfolders:
+                podcast_name = os.path.basename(os.path.dirname(filename))
+            else:
+                podcast_name = None
          
-            t = SyncTrack(title, length, modified, filename=filename)
+            t = SyncTrack(title, length, modified, filename=filename, podcast=podcast_name)
             tracks.append(t)
         return tracks
 
@@ -545,3 +603,40 @@ class MP3PlayerDevice(Device):
         dotfiles = glob.glob(os.path.join(directory, '.*'))
         return len(files+dotfiles) == 0
 
+    def find_scrobbler_log(self, mount_point):
+        """ find an audioscrobbler log file from log_filenames in the mount_point dir """
+        for dirpath, dirnames, filenames in os.walk(mount_point):
+            for log_file in self.scrobbler_log_filenames:
+                filename = os.path.join(dirpath, log_file)
+                if os.path.isfile(filename):
+                    return filename
+
+        # No scrobbler log on that device
+        return None
+
+    def load_audioscrobbler_log(self, log_file):
+        """ Retrive track title and artist info for all the entries
+            in an audioscrobbler portable player format logfile
+            http://www.audioscrobbler.net/wiki/Portable_Player_Logging """
+        try:
+            log('Opening "%s" as AudioScrobbler log.', log_file, sender=self)
+            f = open(log_file, 'r')
+            entries = f.readlines()
+            f.close()
+        except IOError, ioerror:
+            log('Error: "%s" cannot be read.', log_file, sender=self)
+            return False
+
+        try:
+            # regex that can be used to get all the data from a scrobbler.log entry
+            entry_re = re.compile('^(.*)\t(.*)\t(.*)\t(.*)\t(.*)\t(.*)\t(.*)$')
+            for entry in entries:
+                match_obj = re.match(entry_re, entry)
+                # L means at least 50% of the track was listened to (S means < 50%)
+                if match_obj and match_obj.group(6).strip().lower() == 'l':
+                    # append [artist_name, track_name]
+                    self.scrobbler_log.append([match_obj.group(1), match_obj.group(3)])
+        except:
+            log('Error while parsing "%s".', log_file, sender=self)
+
+        return True
