@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (C) 2005-2007 Thomas Perl <thp at perli.net>
+# Copyright (c) 2005-2008 Thomas Perl and the gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@
 import gtk
 import datetime
 
+import gpodder
 from gpodder.liblogger import log
 from gpodder.libgpodder import gl
 from gpodder.libpodcasts import podcastItem
@@ -41,6 +42,9 @@ from gpodder import services
 from gpodder import util
 from gpodder import draw
 
+if gpodder.interface == gpodder.MAEMO:
+    import hildon
+
 class GPodderStatusIcon(gtk.StatusIcon):
     """ this class display a status icon in the system tray
     this icon serves to show or hide gPodder, notify dowload status
@@ -54,8 +58,8 @@ class GPodderStatusIcon(gtk.StatusIcon):
 
     # status: they are displayed as tooltip and add a small icon to the main icon
     STATUS_DOWNLOAD_IN_PROGRESS = (_('Downloading episodes'), gtk.STOCK_GO_DOWN)
-    STATUS_UPDATING_FEED_CACHE = (_('Looking for new episodes'), gtk.STOCK_FIND)
-    STATUS_SYNCHRONIZING = (_('Synchronizing to player'), gtk.STOCK_REFRESH)
+    STATUS_UPDATING_FEED_CACHE = (_('Looking for new episodes'), gtk.STOCK_REFRESH)
+    STATUS_SYNCHRONIZING = (_('Synchronizing to player'), 'multimedia-player')
     STATUS_DELETING = (_('Cleaning files'), gtk.STOCK_DELETE)
 
     # actions: buttons within the notify bubble
@@ -65,11 +69,11 @@ class GPodderStatusIcon(gtk.StatusIcon):
     ACTION_KEEP_DOWLOADING = ('keep_dowloading', _('Keep dowloading'))
     ACTION_START_DOWNLOAD = ('download', _('Download'))
     
-    def __init__(self, gpodder, icon_filename):
+    def __init__(self, gp, icon_filename):
         gtk.StatusIcon.__init__(self)
         log('Creating tray icon', sender=self)
         
-        self.__gpodder = gpodder
+        self.__gpodder = gp
         self.__finished_downloads = []
         self.__icon_cache = {}
         self.__icon_filename = icon_filename
@@ -82,7 +86,10 @@ class GPodderStatusIcon(gtk.StatusIcon):
 
         # try getting the icon
         try:
-            self.__icon = gtk.gdk.pixbuf_new_from_file(self.__icon_filename)
+            if gpodder.interface == gpodder.GUI:
+                self.__icon = gtk.gdk.pixbuf_new_from_file(self.__icon_filename)
+            elif gpodder.interface == gpodder.MAEMO:
+                self.__icon = gtk.gdk.pixbuf_new_from_file_at_size(self.__icon_filename, 36, 36)
         except Exception, exc:
             log('Warning: Cannot load gPodder icon, will use the default icon (%s)', exc, sender=self)
             self.__icon = gtk.icon_theme_get_default().load_icon(gtk.STOCK_DIALOG_QUESTION, 30, 30)
@@ -91,10 +98,15 @@ class GPodderStatusIcon(gtk.StatusIcon):
         self.__current_pixbuf = None
         self.__last_ratio = 1.0
         self.set_status()
-
-        self.connect('activate', self.__on_left_click)
+ 
         menu = self.__create_context_menu()
-        self.connect('popup-menu', self.__on_right_click, menu)
+        if gpodder.interface == gpodder.GUI:
+            self.connect('activate', self.__on_left_click)
+            self.connect('popup-menu', self.__on_right_click, menu)
+        elif gpodder.interface == gpodder.MAEMO:
+            # On Maemo, we show the popup menu on left-click
+            self.connect('activate', self.__on_right_click, menu)
+
         self.set_visible(True)
 
         # initialise pynotify
@@ -148,11 +160,32 @@ class GPodderStatusIcon(gtk.StatusIcon):
         menu.append(menuItem)
         menu.append( gtk.SeparatorMenuItem())
         
+        if gpodder.interface == gpodder.MAEMO:
+            # On Maemo, we map the left-click to the popup-menu,
+            # so add a menu item to do the left-click action
+            menuItem = gtk.ImageMenuItem(_('Hide gPodder'))
+            self.item_showhide = menuItem
+
+            def show_hide_gpodder_maemo(menu_item):
+                visible = self.__gpodder.gPodder.get_property('visible')
+                (label, image) = menu_item.get_children()
+                if visible:
+                    label.set_text(_('Show gPodder'))
+                    menu_item.set_image(gtk.image_new_from_stock(gtk.STOCK_GO_UP, gtk.ICON_SIZE_MENU))
+                else:
+                    label.set_text(_('Hide gPodder'))
+                    menu_item.set_image(gtk.image_new_from_stock(gtk.STOCK_GO_DOWN, gtk.ICON_SIZE_MENU))
+                self.__gpodder.gPodder.set_property('visible', not visible)
+                
+            menuItem.set_image(gtk.image_new_from_stock(gtk.STOCK_GO_DOWN, gtk.ICON_SIZE_MENU))
+            menuItem.connect('activate', lambda widget: show_hide_gpodder_maemo(self.item_showhide))
+            menu.append(menuItem)
+        
         menuItem = gtk.ImageMenuItem(gtk.STOCK_QUIT)
         menuItem.connect('activate',  self.__on_exit_callback)
         menu.append(menuItem)
         
-        return menu        
+        return menu
 
     def __on_exit_callback(self, widget, *args):
         if  self.__is_downloading and self.__is_notification_on():
@@ -165,9 +198,13 @@ class GPodderStatusIcon(gtk.StatusIcon):
         if p != []:
             self.send_notification(p[0], p[1], p[2], p[3])
 
-    def __on_right_click(self, widget, button, time, data=None):
+    def __on_right_click(self, widget, button=None, time=0, data=None):
         """Open popup menu on right-click
         """
+        if gpodder.interface == gpodder.MAEMO and data is None and button is not None:
+            # The left-click action has a different function
+            # signature, so we have to swap parameters here
+            data = button
         if data is not None:
             data.show_all()
             data.popup(None, None, None, 3, time)
@@ -204,17 +241,21 @@ class GPodderStatusIcon(gtk.StatusIcon):
             else:
                 tooltip.append(_('downloading %d episodes')%count)
 
-            tooltip.append(' (%d%%)'%percentage)
+            tooltip.append(' (%d%%) :'%percentage)
+
+            downloading = []
+            for episode in self.__finished_downloads:
+                downloading.append(_("%s (completed)") % episode)
+            for status in services.download_status_manager.status_list.values():
+                downloading.append(status['thread'].episode.title + " (%d%% - %s)" % (status['progress'], status['speed']))
+            tooltip.append(self.format_episode_list(downloading))
 
             if percentage <> 0:
                 date_diff = datetime.datetime.now() - self.__download_start_time
                 estim = date_diff.seconds * 100 // percentage - date_diff.seconds
-                tooltip.append('\n' + _('estimated remaining time: '))
+                tooltip.append('\n' + _('Estimated remaining time: '))
                 tooltip.append(util.format_seconds_to_hour_min_sec(estim))
                 
-            if len(self.__finished_downloads) > 0:
-                tooltip.append(self.format_episode_list(self.__finished_downloads, _('Finished downloads:')))
-
             self.set_status(self.STATUS_DOWNLOAD_IN_PROGRESS, ''.join(tooltip))
             
             self.progress_bar(float(percentage)/100.)
@@ -243,9 +284,10 @@ class GPodderStatusIcon(gtk.StatusIcon):
         try:
             new_icon = self.__icon.copy()
             emblem = gtk.icon_theme_get_default().load_icon(icon, int(new_icon.get_width()/1.5), 0)
-            size = emblem.get_width()
-            pos = new_icon.get_width()-size
-            emblem.composite(new_icon, pos, pos, size, size, pos, pos, 1, 1, gtk.gdk.INTERP_BILINEAR, 255)
+            (width, height) = (emblem.get_width(), emblem.get_height())
+            xpos = new_icon.get_width()-width
+            ypos = new_icon.get_height()-height
+            emblem.composite(new_icon, xpos, ypos, width, height, xpos, ypos, 1, 1, gtk.gdk.INTERP_BILINEAR, 255)
             self.__icon_cache[icon] = new_icon
             return new_icon
         except Exception, exc:
@@ -261,7 +303,7 @@ class GPodderStatusIcon(gtk.StatusIcon):
         if action=='show': 
             self.__gpodder.uniconify_main_window()
         elif action=='quit':
-            util.idle_add(self.__gpodder.close_gpodder)
+            self.__gpodder.close_gpodder()
         elif action=='keep_dowloading':
             pass
         elif action=='force_quit':
@@ -270,6 +312,7 @@ class GPodderStatusIcon(gtk.StatusIcon):
             self.__gpodder.on_itemDownloadAllNew_activate(self.__gpodder)
         else:
             log("don't know what to do with action %s" % action, sender = self)
+        gtk.main_quit()
             
     def __is_notification_on(self):
         # tray icon not visible or notifications disabled
@@ -277,34 +320,43 @@ class GPodderStatusIcon(gtk.StatusIcon):
             return False
         return True
     
-    def destroy(self, n=None, action=None):
-        gtk.main_quit()
- 
     def send_notification( self, message, title = "gPodder", actions = [], is_error=False):
         if not self.__is_notification_on(): return
 
         message = message.strip()
         log('Notification: %s', message, sender=self)
-        if have_pynotify:
+        if gpodder.interface == gpodder.MAEMO:
+            pango_markup = '<b>%s</b>\n<small>%s</small>' % (title, message)
+            hildon.hildon_banner_show_information_with_markup(gtk.Label(''), None, pango_markup)
+        elif gpodder.interface == gpodder.GUI and have_pynotify:
             notification = pynotify.Notification(title, message, self.__icon_filename)
             if is_error: notification.set_urgency(pynotify.URGENCY_CRITICAL)
             try:
                 notification.attach_to_status_icon(self)
             except:
                 log('Warning: Cannot attach notification to status icon.', sender=self)
-            notification.connect('closed', self.destroy)
             for action in actions:
                 notification.add_action(action[0], action[1], self.__action_callback)
             if not notification.show():
                 log("Error: enable to send notification %s", message)
-            self.__previous_notification=[message, title, actions, is_error]
-            self.menuItem_previous_msg.set_sensitive(True)
-            gtk.main()
+            if len(actions) > 0:
+                gtk.main() # needed for action callback to be triggered
+        else:
+            return
+        
+        # If we showed any kind of notification, remember it for next time
+        self.__previous_notification=[message, title, actions, is_error]
+        self.menuItem_previous_msg.set_sensitive(True)
         
     def set_status(self, status=None, tooltip=None):
         if status is None:
             if tooltip is None:
                 tooltip = self.DEFAULT_TOOLTIP
+                if len(self.__gpodder.already_notified_new_episodes) > 0:
+                    tooltip += "\n" + _("New episodes:") 
+                    for episode in self.__gpodder.already_notified_new_episodes:
+                        tooltip += "\n" + episode.title
+                        if episode.is_downloaded(): tooltip +=  _(" (downloaded)")
             else:
                 tooltip = 'gPodder - %s' % tooltip
             if self.__current_icon is not None:
@@ -323,7 +375,7 @@ class GPodderStatusIcon(gtk.StatusIcon):
                 self.__current_icon = icon
         self.set_tooltip(tooltip)
 
-    def format_episode_list(self, episode_list, caption=''):
+    def format_episode_list(self, episode_list, caption=None):
         """
         Format a list of episodes for tooltips and notifications
         Return a listing of episodes title separated by a line break.
@@ -342,7 +394,8 @@ class GPodderStatusIcon(gtk.StatusIcon):
         MAX_TITLE_LENGTH = 100
 
         result = []
-        result.append('\n%s' % caption)
+        if caption is not None:
+            result.append('\n%s' % caption)
         for episode in episode_list[:min(len(episode_list),MAX_EPISODES)]:
             if isinstance(episode, podcastItem): 
                 episode_title = episode.title
