@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (C) 2005-2007 Thomas Perl <thp at perli.net>
+# Copyright (c) 2005-2008 Thomas Perl and the gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -61,6 +61,13 @@ import StringIO
 import xml.dom.minidom
 
 
+if gpodder.interface == gpodder.GUI:
+    ICON_UNPLAYED = gtk.STOCK_YES
+    ICON_LOCKED = 'emblem-nowrite'
+elif gpodder.interface == gpodder.MAEMO:
+    ICON_UNPLAYED = 'qgn_list_gene_favor'
+    ICON_LOCKED = 'qgn_indi_KeypadLk_lock'
+
 def make_directory( path):
     """
     Tries to create a directory if it does not exist already.
@@ -159,11 +166,14 @@ def calculate_size( path):
     if os.path.isdir( path) and not os.path.islink( path):
         sum = os.path.getsize( path)
 
-        for item in os.listdir( path):
-            try:
-                sum += calculate_size( os.path.join( path, item))
-            except:
-                pass
+        try:
+            for item in os.listdir(path):
+                try:
+                    sum += calculate_size(os.path.join(path, item))
+                except:
+                    log('Cannot get size for %s', path)
+        except:
+            log('Cannot access: %s', path)
 
         return sum
 
@@ -241,6 +251,35 @@ def get_free_disk_space(path):
     return s.f_bavail * s.f_bsize
 
 
+def format_date(timestamp):
+    """
+    Converts a UNIX timestamp to a date representation. This
+    function returns "Today", "Yesterday", a weekday name or
+    the date in %x format, which (according to the Python docs)
+    is the "Locale's appropriate date representation".
+
+    Returns None if there has been an error converting the
+    timestamp to a string representation.
+    """
+    seconds_in_a_day = 60*60*24
+    try:
+        diff = int((time.time()+1)/seconds_in_a_day) - int(timestamp/seconds_in_a_day)
+    except:
+        log('Warning: Cannot convert "%s" to date.', timestamp, traceback=True)
+        return None
+    
+    if diff == 0:
+       return _('Today')
+    elif diff == 1:
+       return _('Yesterday')
+    elif diff < 7:
+        # Weekday name
+        return str(datetime.datetime.fromtimestamp(timestamp).strftime('%A'))
+    else:
+        # Locale's appropriate date representation
+        return str(datetime.datetime.fromtimestamp(timestamp).strftime('%x'))
+
+
 def format_filesize(bytesize, use_si_units=False, digits=2):
     """
     Formats the given size in bytes to be human-readable, 
@@ -321,7 +360,7 @@ def remove_html_tags(html):
     # Convert named HTML entities to their unicode character
     result = re_html_entities.sub(lambda x: unicode(entitydefs.get(x.group(1),''), 'iso-8859-1'), result)
 
-    return result
+    return result.strip()
 
 
 def torrent_filename( filename):
@@ -368,7 +407,7 @@ def file_extension_from_url( url):
     filename = os.path.basename( urllib.unquote(path))
     (filename, extension) = os.path.splitext(filename)
 
-    if file_type_by_extension(extension) != None:
+    if file_type_by_extension(extension) is not None:
         # We have found a valid extension (audio, video, torrent)
         return extension.lower()
     
@@ -377,7 +416,7 @@ def file_extension_from_url( url):
         query_url = '://'.join((scheme, urllib.unquote(query)))
         query_extension = file_extension_from_url(query_url)
 
-        if file_type_by_extension(query_extension) != None:
+        if file_type_by_extension(query_extension) is not None:
             return query_extension
 
     # No exact match found, simply return the original extension
@@ -427,8 +466,9 @@ def get_tree_icon(icon_name, add_bullet=False, add_padlock=False, icon_cache=Non
     the cache is supplied again and the icon is found in 
     the cache.
     """
+    global ICON_UNPLAYED, ICON_LOCKED
 
-    if icon_cache != None and (icon_name,add_bullet,add_padlock,icon_size) in icon_cache:
+    if icon_cache is not None and (icon_name,add_bullet,add_padlock,icon_size) in icon_cache:
         return icon_cache[(icon_name,add_bullet,add_padlock,icon_size)]
     
     icon_theme = gtk.icon_theme_get_default()
@@ -444,22 +484,23 @@ def get_tree_icon(icon_name, add_bullet=False, add_padlock=False, icon_cache=Non
         if add_bullet:
             try:
                 icon = icon.copy()
-                emblem = icon_theme.load_icon(gtk.STOCK_YES, int(float(icon_size)*1.2/3.0), 0)
-                size = emblem.get_width()
-                pos = icon.get_width() - size
-                emblem.composite(icon, pos, pos, size, size, pos, pos, 1, 1, gtk.gdk.INTERP_BILINEAR, 255)
+                emblem = icon_theme.load_icon(ICON_UNPLAYED, int(float(icon_size)*1.2/3.0), 0)
+                (width, height) = (emblem.get_width(), emblem.get_height())
+                xpos = icon.get_width() - width
+                ypos = icon.get_height() - height
+                emblem.composite(icon, xpos, ypos, width, height, xpos, ypos, 1, 1, gtk.gdk.INTERP_BILINEAR, 255)
             except:
                 log('(get_tree_icon) Error adding emblem to icon "%s".', icon_name)
         if add_padlock:
             try:
                 icon = icon.copy()
-                emblem = icon_theme.load_icon('emblem-nowrite', int(float(icon_size)/2.0), 0)
-                size = emblem.get_width()
-                emblem.composite(icon, 0, 0, size, size, 0, 0, 1, 1, gtk.gdk.INTERP_BILINEAR, 255)
+                emblem = icon_theme.load_icon(ICON_LOCKED, int(float(icon_size)/2.0), 0)
+                (width, height) = (emblem.get_width(), emblem.get_height())
+                emblem.composite(icon, 0, 0, width, height, 0, 0, 1, 1, gtk.gdk.INTERP_BILINEAR, 255)
             except:
                 log('(get_tree_icon) Error adding emblem to icon "%s".', icon_name)
 
-    if icon_cache != None:
+    if icon_cache is not None:
         icon_cache[(icon_name,add_bullet,add_padlock,icon_size)] = icon
 
     return icon
@@ -477,11 +518,22 @@ def updated_parsed_to_rfc2822( updated_parsed):
     """
     Converts a 9-tuple from feedparser's updated_parsed 
     field to a C-locale string suitable for further use.
+
+    If the updated_parsed field is None or not a 9-tuple,
+    this function returns None.
     """
+    if updated_parsed is None or len(updated_parsed) != 9:
+        return None
+
     old_locale = locale.getlocale( locale.LC_TIME)
     locale.setlocale( locale.LC_TIME, 'C')
     result = time.strftime( '%a, %d %b %Y %H:%M:%S GMT', updated_parsed)
-    locale.setlocale( locale.LC_TIME, old_locale)
+    if old_locale != (None, None):
+        try:
+            locale.setlocale( locale.LC_TIME, old_locale)
+        except:
+            log('Cannot revert locale to (%s, %s)', *old_locale)
+            pass
     return result
 
 
@@ -573,7 +625,11 @@ def parse_itunes_xml(url):
     """
     url = url.replace('itms://', 'http://')
     doc = http_get_and_gunzip(url)
-    d = xml.dom.minidom.parseString(doc)
+    try:
+        d = xml.dom.minidom.parseString(doc)
+    except Exception, e:
+        log('Error parsing document from itms:// URL: %s', e)
+        return None
     last_key = None
     for pairs in d.getElementsByTagName('dict'):
         for node in pairs.childNodes:
@@ -622,6 +678,9 @@ def itunes_discover_rss(url):
     Idea from Andrew Clarke's itunes-url-decoder.py
     """
 
+    if url is None:
+        return url
+
     if not 'phobos.apple.com' in url.lower():
         # This doesn't look like an iTunes URL
         return url
@@ -644,7 +703,7 @@ def idle_add(func, *args):
     call the function later - this is needed for
     threads to be able to modify GTK+ widget data.
     """
-    if gpodder.interface_is_gui:
+    if gpodder.interface in (gpodder.GUI, gpodder.MAEMO):
         def x(f, *a):
             f(*a)
             return False
@@ -865,3 +924,20 @@ def sanitize_filename(filename):
 
     return re.sub('[/|?*<>:+\[\]\"\\\]', '_', filename.strip().encode(enc, 'ignore'))
 
+
+def find_mount_point(directory):
+    """
+    Try to find the mount point for a given directory.
+    If the directory is itself a mount point, return
+    it. If not, remove the last part of the path and
+    re-check if it's a mount point. If the directory
+    resides on your root filesystem, "/" is returned.
+    """
+    while os.path.split(directory)[0] != '/':
+        if os.path.ismount(directory):
+            return directory
+        else:
+            (directory, tail_data) = os.path.split(directory)
+
+    return '/'
+
